/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.framework.pay.core;

import com.alibaba.fastjson.JSONObject;
import com.framework.pay.core.base.PayService;
import com.framework.pay.core.domain.PayMark;
import com.framework.pay.core.enums.ServiceExceptionEnum;
import com.framework.pay.core.exception.PayServiceException;
import com.framework.pay.core.toolkit.ApplicationContextHolder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.context.event.ApplicationContextInitializedEvent;
import org.springframework.context.ApplicationListener;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @Filename: PayFactory
 * @Author: zzy
 * @Data:2023/4/28 21:45
 * @Description: 获取实际支付类
 */

@Slf4j
public class PayFactory implements ApplicationListener<ApplicationContextInitializedEvent> {

    /**
     * 支付抽象实现缓存对象
     */
    private final ConcurrentHashMap<PayMark, PayService> payConcurrentHashMap = new ConcurrentHashMap<>();


    /**
     * 获取对应支付实现
     *
     * @param payMark   支付实现标记
     * @return          实际支付实现
     */
    public PayService getInstance(PayMark payMark){
        //Optional.ofNullable(payConcurrentHashMap).orElseThrow(() -> new PayServiceException(ServiceExceptionEnum.PAY_CONCURRENTMAP_IS_NULL));
        if(!payConcurrentHashMap.contains(payMark)){
           log.info("PayMark.payType:{}" + payMark.getPayType()+"PayMark.payTradeChannel:{}" + payMark.getPayTradeChannel() + "not find suitable payService");
           throw new PayServiceException(ServiceExceptionEnum.PAY_NO_FIND_SUITABLE_PAYSERVICE);
        }
       return payConcurrentHashMap.get(payMark);
    }

    /**
     * 应用初始化事件
     *
     * @param applicationContextInitializedEvent    应用上下文事件
     */
    @Override
    public void onApplicationEvent(ApplicationContextInitializedEvent applicationContextInitializedEvent) {
        Map<String, PayService> beansMap = ApplicationContextHolder.getBeansOfType((PayService.class));
        beansMap.forEach((name,bean)->{
            PayService payService = payConcurrentHashMap.get(bean.mark());
            if (payService != null) {
                throw new PayServiceException(String.format("[%s] Duplicate execution policy", bean.mark()));
            }
            payConcurrentHashMap.put(bean.mark(),bean);
        });
        log.info("payConcurrentHashMap:{}" + JSONObject.toJSONString(payConcurrentHashMap));
    }
}
