/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.framework.pay.core.handler.alipay;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.request.AlipayTradeQueryRequest;
import com.alipay.api.request.AlipayTradeRefundRequest;
import com.alipay.api.request.AlipayTradeWapPayRequest;
import com.alipay.api.response.AlipayTradeQueryResponse;
import com.alipay.api.response.AlipayTradeRefundResponse;
import com.alipay.api.response.AlipayTradeWapPayResponse;
import com.framework.pay.core.base.PayService;
import com.framework.pay.core.base.param.*;
import com.framework.pay.core.domain.PayMark;
import com.framework.pay.core.enums.AlipayRespCodeEnum;
import com.framework.pay.core.enums.PayTradeChannel;
import com.framework.pay.core.enums.PayType;
import com.framework.pay.core.enums.ServiceExceptionEnum;
import com.framework.pay.core.exception.PayServiceException;
import com.framework.pay.core.handler.alipay.config.AliPayProperties;
import com.framework.pay.core.handler.alipay.config.AlipayClientComponent;
import com.framework.pay.core.handler.alipay.filter.AlipayParamFilter;
import com.framework.pay.core.handler.alipay.filter.AlipayQueryParamFilter;
import com.framework.pay.core.handler.alipay.filter.AlipayRefundParamFilter;
import com.framework.pay.core.handler.alipay.param.*;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;

/**
 * @Filename: AlipayWapServiceImpl
 * @Author: zzy
 * @Data:2023/4/28 22:58
 * @Description: 阿里手机网站支付实现类
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class AlipayWapServiceImpl implements PayService {

    private final AlipayClientComponent alipayClientComponent;

    private final AlipayQueryParamFilter alipayQueryParamFilter;

    private final AlipayParamFilter alipayParamFilter;

    private final AlipayRefundParamFilter alipayRefundParamFilter;

    private final AliPayProperties aliPayProperties;

    @Override
    public AbstractResponseParam pay(AbstractPayParam payParam) {
        log.info("AlipayWapServiceImpl.pay.param:{}" + JSONObject.toJSONString(payParam));
        AlipayParam alipayParam = alipayParamFilter.filter(payParam);
        AlipayTradeWapPayRequest payRequest = new AlipayTradeWapPayRequest();
        payRequest.setNotifyUrl(Optional.ofNullable(alipayParam.getNotifyUrl()).orElse(aliPayProperties.getNotifyUrl()));
        payRequest.setReturnUrl(alipayParam.getReturnUrl());
        payRequest.setBizContent(buildPayBizContent(alipayParam).toString());
        log.info("AlipayWapServiceImpl.AlipayTradeWapPayRequest.param:{}" + JSONObject.toJSONString(payRequest));
        try {
            AlipayTradeWapPayResponse alipayTradeWapPayResponse = alipayClientComponent.getAlipayClient().pageExecute(payRequest);
            log.info("AlipayWapServiceImpl.pay.response:{}" + JSONObject.toJSONString(alipayTradeWapPayResponse));
            return AlipayResponseParam.builder().body(alipayTradeWapPayResponse.getBody())
                    .success(alipayTradeWapPayResponse.isSuccess())
                    .build();
        } catch (AlipayApiException e) {
            log.info("AlipayWapServiceImpl.pay occured error");
            e.printStackTrace();
            throw new PayServiceException(ServiceExceptionEnum.ALIPAY_WAP_PAY_OCCURED_ERROR);
        }
    }

    @Override
    public AbstractResponseParam tradeQuery(AbstractQueryParam queryParam) {
        log.info("AlipayWapServiceImpl.tradeQuery.param:{}" + JSONObject.toJSONString(queryParam));
        AlipayQueryParam alipayQueryParam = alipayQueryParamFilter.filter(queryParam);
        AlipayTradeQueryRequest request = new AlipayTradeQueryRequest();
        JSONObject paramJson = new JSONObject();
        if (StringUtils.isNotEmpty(alipayQueryParam.getTradeNo())) {
            paramJson.put("trade_no", alipayQueryParam.getTradeNo());
        }
        if (StringUtils.isNotEmpty(alipayQueryParam.getOutTradeNo())) {
            paramJson.put("out_trade_no", alipayQueryParam.getOutTradeNo());
        }
        request.setBizContent(paramJson.toString());
        try {
            AlipayTradeQueryResponse response = alipayClientComponent.getAlipayClient().execute(request);
            log.info("AlipayWapServiceImpl.tradeQuery.response:{}" + JSONObject.toJSONString(response));
            return buildAlipayQueryResponseParam(response);
        } catch (AlipayApiException e) {
            log.info("AlipayWapServiceImpl.tradeQuery occured error");
            e.printStackTrace();
            throw new PayServiceException(ServiceExceptionEnum.ALIPAY_QUERY_OCCURED_ERROR);
        }
    }

    @Override
    public AbstractResponseParam refund(AbstractRefundParam refundParam) throws PayServiceException {
        log.info("AlipayWapServiceImpl.refund.param:{}" + JSONObject.toJSONString(refundParam));
        AlipayRefundParam alipayRefundParam = alipayRefundParamFilter.filter(refundParam);
        AlipayTradeRefundRequest request = new AlipayTradeRefundRequest();
        JSONObject bizContent = new JSONObject();
        bizContent.put("trade_no", alipayRefundParam.getTradeNo());
        bizContent.put("out_trade_no", alipayRefundParam.getOutTradeNo());
        bizContent.put("refund_amount", alipayRefundParam.getRefundAmount());
        if (StringUtils.isNotEmpty(alipayRefundParam.getRefundReason())) {
            bizContent.put("refund_reason", alipayRefundParam.getRefundReason());
        }
        request.setBizContent(bizContent.toString());
        try {
            AlipayTradeRefundResponse refundResponse = alipayClientComponent.getAlipayClient().execute(request);
            return buildAlipayRefundResponseParam(refundResponse);
        } catch (AlipayApiException e) {
            log.info("AlipayWapServiceImpl.refund occured error");
            e.printStackTrace();
            throw new PayServiceException(ServiceExceptionEnum.ALIPAY_REFUND_OCCURED_ERROR);
        }
    }


    @Override
    public void callBack(AbstractCallBackParam abstractCallBackParam) {

    }

    @Override
    public PayMark mark() {
        return PayMark.builder().payType(PayType.ALIPAY).payTradeChannel(PayTradeChannel.WAP).build();
    }

    /**
     * AlipayTradeQueryResponse转化为AlipayQueryResponseParam对象
     *
     * @param response
     * @return
     */
    private AlipayQueryResponseParam buildAlipayQueryResponseParam(AlipayTradeQueryResponse response) {
        AlipayQueryResponseParam queryResponseParam = AlipayQueryResponseParam.builder()
                .code(AlipayRespCodeEnum.getAlipayRespCodeEnum(response.getCode()))
                .message(response.getMsg())
                .subCode(response.getSubCode())
                .subMsg(response.getSubMsg())
                .build();
        if (Objects.equals(queryResponseParam.getCode(), AlipayRespCodeEnum.SUCCESS)) {
            //todo 待补充
            return queryResponseParam;
        }
        return queryResponseParam;
    }

    /**
     * AlipayTradeRefundResponse转化为AlipayRefundResponseParam对象
     *
     * @param response
     * @return
     */
    private AlipayRefundResponseParam buildAlipayRefundResponseParam(AlipayTradeRefundResponse response) {
        AlipayRefundResponseParam refundResponseParam = AlipayRefundResponseParam.builder()
                .code(AlipayRespCodeEnum.getAlipayRespCodeEnum(response.getCode()))
                .message(response.getMsg())
                .subCode(response.getSubCode())
                .subMsg(response.getSubMsg())
                .build();
        if (Objects.equals(refundResponseParam.getCode(), AlipayRespCodeEnum.SUCCESS)) {
            refundResponseParam.setTradeNo(response.getTradeNo());
            refundResponseParam.setOutTradeNo(response.getOutTradeNo());
            refundResponseParam.setFundChange(response.getFundChange());
            refundResponseParam.setRefundFee(new BigDecimal(response.getRefundFee()));
        }
        return refundResponseParam;
    }

    /**
     * 构建阿里支付入参
     *
     * @param param
     * @return
     */
    private JSONObject buildPayBizContent(AlipayParam param) {
        JSONObject bizObject = new JSONObject();
        bizObject.put("out_trade_no", param.getOutTraderNo());
        bizObject.put("total_amount", param.getPrice());
        bizObject.put("subject", param.getSubject());
        if (StringUtils.isNotEmpty(param.getProductCode())) {
            bizObject.put("product_code", param.getProductCode());
        }
        if (null != param.getTransCurrency()) {
            bizObject.put("trans_currency", param.getTransCurrency().toString());
        }
        if (StringUtils.isNotEmpty(param.getCallBackBeanName()) && StringUtils.isNotEmpty(param.getCallBackMethodName())) {
            Map<String, String> passbackParams = new HashMap<>();
            passbackParams.put("callBackBeanName", param.getCallBackBeanName());
            passbackParams.put("callBackMethodName", param.getCallBackBeanName());
            bizObject.put("passback_params", JSONObject.toJSONString(passbackParams));
        }
        return bizObject;
    }
}
