/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.framework.wxpay.client.core.config;

import com.framework.wxpay.client.core.domain.SignatureResult;

import java.security.*;
import java.util.Base64;

import static java.util.Objects.requireNonNull;


/**
 * @author zzy
 * @date 2023/5/12-16:05
 * @Description 签名方法抽象实现
 */
public abstract class AbstractSigner implements Signer {

    /**
     * 签名算法
     */
    protected String algorithmName;

    /**
     * 商户api证书序列号
     */
    protected String certificateSerialNumber;

    /**
     * 签名算法
     */
    protected String algorithm;

    /**
     * 商户API私钥
     */
    protected PrivateKey privateKey;

    public AbstractSigner(String algorithmName, String certificateSerialNumber, String algorithm, PrivateKey privateKey) {
        this.algorithmName = requireNonNull(algorithmName);
        this.certificateSerialNumber = requireNonNull(certificateSerialNumber);
        this.algorithm = requireNonNull(algorithm);
        this.privateKey = requireNonNull(privateKey);
    }

    @Override
    public String getAlgorithm() {
        return algorithm;
    }

    @Override
    public SignatureResult sign(String message) {
        requireNonNull(message);
        byte[] sign;
        try {
            //根据算法名称获取对应签名实例
            Signature pSignature = Signature.getInstance(algorithmName);
            pSignature.initSign(privateKey);
            pSignature.update(message.getBytes());
            sign = pSignature.sign();
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
            throw new UnsupportedOperationException(
                    "The current Java environment does not support " + algorithmName, e);
        } catch (InvalidKeyException e) {
            e.printStackTrace();
            throw new IllegalArgumentException(algorithm + " signature uses an illegal privateKey.", e);
        } catch (SignatureException e) {
            e.printStackTrace();
            throw new RuntimeException("An error occurred during the sign process.", e);
        }
        return new SignatureResult(Base64.getEncoder().encodeToString(sign), certificateSerialNumber);
    }
}
