/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.framework.wxpay.client.core.config;

import com.framework.wxpay.client.core.domain.SignatureResult;
import com.framework.wxpay.client.core.toolkit.NonceUtil;

import java.net.URI;
import java.time.Instant;

import static java.util.Objects.requireNonNull;

/**
 * @author zzy
 * @date 2023/5/12-16:27
 * @Description 微信支付抽象认证方法实现
 */
public class WechatCredentital implements Credential{

    /**
     * 商户号
     */
    private String merchanId;

    /**
     * 微信v3支付 认证类型前缀
     */
    public static final String SCHEMA_PREFIX = "WECHATPAY2-";

    /**
     * 生成随机数长度
     */
    private static final int NONCE_LENGTH = 32;


    /**
     * 签名算法实现
     */
    private Signer signer;

    public WechatCredentital(String merchanId, Signer signer) {
        this.merchanId = requireNonNull(merchanId);
        this.signer = requireNonNull(signer);
    }

    @Override
    public String getAuthorization(URI uri, String httpMethod, String signMessage) {
        requireNonNull(uri);
        requireNonNull(httpMethod);
        return getSchema() + " " + getToken(uri,httpMethod,signMessage);
    }


    /**
     * 生成微信支付认证token
     *
     * @param uri
     * @param httpMethod
     * @param signMessage
     * @return
     */
    private String getToken(URI uri, String httpMethod, String signMessage){
        //获取随机数
        String nonceStr = NonceUtil.createNonce(NONCE_LENGTH);
        //获取时间戳
        long timestamp = Instant.now().getEpochSecond();
        //生成签名
        SignatureResult signature = signer.sign(signMessage);
        //拼接生成token字符串
        String token =
                "mchid=\""
                        + getMerchantId()
                        + "\","
                        + "nonce_str=\""
                        + nonceStr
                        + "\","
                        + "timestamp=\""
                        + timestamp
                        + "\","
                        + "serial_no=\""
                        + signature.getCertificateSerialNumber()
                        + "\","
                        + "signature=\""
                        + signature.getSignature()
                        + "\"";
        return token;
    }

    @Override
    public String getMerchantId() {
        return merchanId;
    }

    @Override
    public String getSchema() {
        return SCHEMA_PREFIX + signer.getAlgorithm();
    }
}
