/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.framework.wxpay.client.core.toolkit;

import java.io.InputStream;
import java.security.KeyPair;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.cert.X509Certificate;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;

/**
 * @author zzy
 * @date 2023/5/12-17:03
 */

public class KeyPairUtil {

    private static KeyStore keyStore;

    private static Object LOCK = new Object();

    /**
     * 获取公私钥.
     *
     * @param keyPath  the key path
     * @param keyAlias the key alias
     * @param keyPass  password
     * @return the key pair
     */
    public static KeyPair getKeyPair(String keyPath, String keyAlias, String keyPass) {
        char[] pem = keyPass.toCharArray();
        try {
            synchronized (LOCK) {
                if (keyStore == null) {
                    synchronized (keyStore) {
                        Resource resource = new ClassPathResource(keyPath);
                        InputStream in = resource.getInputStream();
                        keyStore = KeyStore.getInstance("PKCS12");
                        keyStore.load(in, pem);
                    }
                }
            }
            X509Certificate certificate = (X509Certificate) keyStore.getCertificate(keyAlias);
            certificate.checkValidity();
            // 证书的序列号 也有用
            String serialNumber = certificate.getSerialNumber().toString(16).toUpperCase();
            // 证书的 公钥
            PublicKey publicKey = certificate.getPublicKey();
            // 证书的私钥
            PrivateKey storeKey = (PrivateKey) keyStore.getKey(keyAlias, pem);
            return new KeyPair(publicKey, storeKey);

        } catch (Exception e) {
            throw new IllegalStateException("Cannot load keys from store: ", e);
        }
    }
}
