/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.framework.design.structure.adapter;

import java.lang.reflect.Constructor;

/**
 * @Filename: LogFactory
 * @Author: zzy
 * @Date: 2023/5/24-17:42
 * @Description: 日志工厂
 */

public class LogFactory {

    /**
     * 构造器
     */
    private static Constructor<? extends Log> logConstructor;


    /**
     * 私有构造方法
     */
    private LogFactory() {

    }

    public static Constructor<? extends Log> getLogConstructor() {
        return logConstructor;
    }

    /**
     * 获取log实例
     *
     * @param clazz
     * @return
     * @throws Exception
     */
    private static Log getLogger(Class<?> clazz) throws Exception {
        return getLog(clazz.getName());
    }

    /**
     * 根据log名称获取log实例
     *
     * @param logger
     * @return
     * @throws Exception
     */
    public static Log getLog(String logger) throws Exception {
        try {
            return (Log) logConstructor.newInstance(logger);
        } catch (Throwable var2) {
            throw new Exception("Error creating logger for logger " + logger + ".  Cause: " + var2, var2);
        }
    }


    public static synchronized void useSlf4jLogging() {
        setImplementation(NativeSlf4jLogImpl.class);
    }

    private static void setImplementation(Class<? extends Log> implClass) {
        try {
            Constructor<? extends Log> candidate = implClass.getConstructor();
            Log log = (Log) candidate.newInstance();
            if (log.isDebugEnabled()) {
                log.debug("Logging initialized using '" + implClass + "' adapter.");
            }

            logConstructor = candidate;
        } catch (Throwable var3) {

        }
    }

    private static void tryImplementation(Runnable runnable) {
        if (logConstructor == null) {
            try {
                runnable.run();
            } catch (Throwable var2) {
                ;
            }
        }

    }

    static {
        tryImplementation(LogFactory::useSlf4jLogging);
    }
}
