/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.framework.toolkit;

import com.alipay.api.internal.util.StreamUtil;
import com.alipay.api.java_websocket.util.Base64;
import org.springframework.util.StringUtils;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.security.KeyFactory;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;

/**
 * @Filename:
 * @Author: zzy
 * @Date: 2023/6/5-11:10
 * @Description: RSA 签名生成、校验、加解密具体实现
 */
public class RSAEncryptor extends BaseAsymmetricEncryptor {

    @Override
    protected String doSign(String sourceContent, String privateKey, String charset) throws Exception {
       //获取privateKey
        PrivateKey key = getPrivateKeyFromPKCS8("RSA", new ByteArrayInputStream(privateKey.getBytes()));
        Signature signature = Signature.getInstance(this.getAlgorithm());
        signature.initSign(key);
        if(StringUtils.isEmpty(charset)){
            signature.update(sourceContent.getBytes());
        }else{
            signature.update(sourceContent.getBytes(charset));
        }
        byte[] bySign = signature.sign();
        return new String(Base64.encodeBytes(bySign));
    }

    @Override
    protected boolean doVerify(String sourceContent, String publicKey, String charset, String sign) throws Exception {
        PublicKey pubKey = getPublicKeyFromX509("RSA", new ByteArrayInputStream(publicKey.getBytes()));
        Signature signature = Signature.getInstance(this.getAlgorithm());
        signature.initVerify(pubKey);
        if (com.alipay.api.internal.util.StringUtils.isEmpty(charset)) {
            signature.update(sourceContent.getBytes());
        } else {
            signature.update(sourceContent.getBytes(charset));
        }
        return signature.verify(com.alipay.api.internal.util.codec.Base64.decodeBase64(sign.getBytes()));
    }

    @Override
    public String encrypt(String plainText, String charset, String publicKey) {
        return null;
    }

    @Override
    public String decrypt(String cipherText, String charset, String privateKey) {
        return null;
    }

    protected String getAlgorithm(){
        return "SHA1WithRSA";
    }

    public static PrivateKey getPrivateKeyFromPKCS8(String algorithm, InputStream ins) throws Exception {
        if (ins != null && !StringUtils.isEmpty(algorithm)) {
            KeyFactory keyFactory = KeyFactory.getInstance(algorithm);
            byte[] encodedKey = StreamUtil.readText(ins).getBytes();
            encodedKey = com.alipay.api.internal.util.codec.Base64.decodeBase64(encodedKey);
            return keyFactory.generatePrivate(new PKCS8EncodedKeySpec(encodedKey));
        } else {
            return null;
        }
    }

    public static PublicKey getPublicKeyFromX509(String algorithm, InputStream ins) throws Exception {
        KeyFactory keyFactory = KeyFactory.getInstance(algorithm);
        StringWriter writer = new StringWriter();
        StreamUtil.io(new InputStreamReader(ins), writer);
        byte[] encodedKey = writer.toString().getBytes();
        encodedKey = com.alipay.api.internal.util.codec.Base64.decodeBase64(encodedKey);
        return keyFactory.generatePublic(new X509EncodedKeySpec(encodedKey));
    }
}
