/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.framework.junit.wechat;

import com.alibaba.fastjson.JSONObject;
import com.framework.junit.http.okhttp.*;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;

import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.security.*;
import java.security.cert.X509Certificate;
import java.time.Instant;
import java.util.Base64;

import static java.util.Objects.requireNonNull;

/**
 * @author zzy
 * @date 2023/5/12-11:40
 */

public class WechatV3Test {

    private static KeyStore store;

    private final static Object lock = new Object();

    private static final int NONCE_LENGTH = 32;
    public static final String SCHEMA_PREFIX = "WECHATPAY2-";

    public static void main(String args[]) throws Exception {
        KeyPair keyPair = createPKCS12("apiclient_uk_cert.p12", "Tenpay Certificate", "1512186831");

        String requestPath = "https://api.mch.weixin.qq.com/v3/certificates";
        requestPath = requestPath.replaceFirst(HostName.API.getValue(), "");
        URL url = new URL(requestPath);
        URI uri = url.toURI();

        PrepayRequest request = new PrepayRequest();
        // 调用request.setXxx(val)设置所需参数，具体参数可见Request定义
        // 调用接口

        request.setAppid("wxd8bf812eebe3f58b");
        Amount amount = new Amount();
        amount.setCurrency("CNY");
        amount.setTotal(1);
        request.setAmount(amount);
        request.setDescription("测试");
        request.setMchid("1512186831");
        request.setOutTradeNo("1111111111111111");
        request.setNotifyUrl("https://www.xwsoft.com.cn/freesim/wxH5/wxH5PayNotify");
        SceneInfo sceneInfo = new SceneInfo();
        sceneInfo.setPayerClientIp("127.0.0.1");
        request.setSceneInfo(sceneInfo);

        String httpMethod = "GET";
        String signBody = JSONObject.toJSONString(request);

        String nonceStr = NonceUtil.createNonce(NONCE_LENGTH);
        long timestamp = Instant.now().getEpochSecond();
        String message = buildMessage(nonceStr, timestamp, uri, httpMethod, "");
        System.out.println(message);
        RSASigner rsaSigner = new RSASigner("639A85CC5198322F96B2E1DA09C6A12077BABA75", keyPair.getPrivate());

        SignatureResult signature = rsaSigner.sign(message);
        String token =
                "mchid=\""
                        + "1512186831"
                        + "\","
                        + "nonce_str=\""
                        + nonceStr
                        + "\","
                        + "timestamp=\""
                        + timestamp
                        + "\","
                        + "serial_no=\""
                        + "639A85CC5198322F96B2E1DA09C6A12077BABA75"
                        + "\","
                        + "signature=\""
                        + signature.getSign()
                        + "\"";


        System.out.println(token);

        System.out.println(SCHEMA_PREFIX + rsaSigner.getAlgorithm()+" " + token);

    }


    private static String buildMessage(
            String nonce, long timestamp, URI uri, String httpMethod, String signBody) {
        String canonicalUrl = uri.getRawPath();
        if (uri.getQuery() != null) {
            canonicalUrl += "?" + uri.getRawQuery();
        }
        System.out.println(canonicalUrl);
        return httpMethod
                + "\n"
                + canonicalUrl
                + "\n"
                + timestamp
                + "\n"
                + nonce
                + "\n"
                + (signBody == null ? "" : signBody)
                + "\n";
    }


    /**
     * 获取公私钥.
     *
     * @param keyPath  the key path
     * @param keyAlias the key alias
     * @param keyPass  password
     * @return the key pair
     */
    public static KeyPair createPKCS12(String keyPath, String keyAlias, String keyPass) {
        char[] pem = keyPass.toCharArray();
        try {
            synchronized (lock) {
                if (store == null) {
                    synchronized (lock) {
                        Resource resource = new ClassPathResource(keyPath);
                        InputStream in = resource.getInputStream();
                        store = KeyStore.getInstance("PKCS12");
                        store.load(in, pem);
                    }
                }
            }
            X509Certificate certificate = (X509Certificate) store.getCertificate(keyAlias);
            certificate.checkValidity();
            // 证书的序列号 也有用
            String serialNumber = certificate.getSerialNumber().toString(16).toUpperCase();
            // 证书的 公钥
            PublicKey publicKey = certificate.getPublicKey();
            // 证书的私钥
            PrivateKey storeKey = (PrivateKey) store.getKey(keyAlias, pem);

            return new KeyPair(publicKey, storeKey);

        } catch (Exception e) {
            throw new IllegalStateException("Cannot load keys from store: ", e);
        }
    }

}
